@@ -50,11 +50,9 @@ totem_pl_parser_is_rss (const char *data, gsize len)
 	if (len > MIME_READ_CHUNK_SIZE)
 		len = MIME_READ_CHUNK_SIZE;
 
-	if (memmem (data, len,
-		    RSS_NEEDLE, strlen (RSS_NEEDLE)) != NULL)
+	if (g_strstr_len (data, len, RSS_NEEDLE) != NULL)
 		return RSS_MIME_TYPE;
-	if (memmem (data, len,
-		    RSS_NEEDLE2, strlen (RSS_NEEDLE2)) != NULL)
+	if (g_strstr_len (data, len, RSS_NEEDLE2) != NULL)
 		return RSS_MIME_TYPE;
 
 	return NULL;
@@ -68,8 +66,7 @@ totem_pl_parser_is_atom (const char *data, gsize len)
 	if (len > MIME_READ_CHUNK_SIZE)
 		len = MIME_READ_CHUNK_SIZE;
 
-	if (memmem (data, len,
-		    ATOM_NEEDLE, strlen (ATOM_NEEDLE)) != NULL)
+	if (g_strstr_len (data, len, ATOM_NEEDLE) != NULL)
 		return ATOM_MIME_TYPE;
 
 	return NULL;
@@ -83,8 +80,7 @@ totem_pl_parser_is_opml (const char *data, gsize len)
 	if (len > MIME_READ_CHUNK_SIZE)
 		len = MIME_READ_CHUNK_SIZE;
 
-	if (memmem (data, len,
-		    OPML_NEEDLE, strlen (OPML_NEEDLE)) != NULL)
+	if (g_strstr_len (data, len, OPML_NEEDLE) != NULL)
 		return OPML_MIME_TYPE;
 
 	return NULL;
@@ -246,6 +242,7 @@ totem_pl_parser_add_rss (TotemPlParser *parser,
 	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
+	totem_pl_parser_cleanup_xml (contents);
 	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
 	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
 		g_free (contents);
@@ -287,11 +284,15 @@ totem_pl_parser_add_itpc (TotemPlParser *parser,
 			  gpointer data)
 {
 	TotemPlParserResult ret;
-	char *new_url;
+	char *url, *new_url, *uri_scheme;
 	GFile *new_file;
 
-	new_url = g_file_get_uri (file);
-	memcpy (new_url, "http", 4);
+	url = g_file_get_uri (file);
+	uri_scheme = g_file_get_uri_scheme (file);
+	new_url = g_strdup_printf ("http%s", url + strlen (uri_scheme));
+	g_free (url);
+	g_free (uri_scheme);
+
 	new_file = g_file_new_for_uri (new_url);
 	g_free (new_url);
 
@@ -302,6 +303,40 @@ totem_pl_parser_add_itpc (TotemPlParser *parser,
 	return ret;
 }
 
+TotemPlParserResult
+totem_pl_parser_add_zune (TotemPlParser *parser,
+			  GFile *file,
+			  GFile *base_file,
+			  gpointer data)
+{
+	TotemPlParserResult ret;
+	char *url, *new_url;
+	GFile *new_file;
+
+	url = g_file_get_uri (file);
+	if (g_str_has_prefix (url, "zune://subscribe/?") == FALSE) {
+		g_free (url);
+		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	}
+
+	new_url = strchr (url + strlen ("zune://subscribe/?"), '=');
+	if (new_url == NULL) {
+		g_free (url);
+		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	}
+	/* Skip over the '=' */
+	new_url++;
+
+	new_file = g_file_new_for_uri (new_url);
+	g_free (url);
+
+	ret = totem_pl_parser_add_rss (parser, new_file, base_file, data);
+
+	g_object_unref (new_file);
+
+	return ret;
+}
+
 /* Atom docs:
  * http://www.atomenabled.org/developers/syndication/atom-format-spec.php#rfc.section.4.1
  * http://tools.ietf.org/html/rfc4287
@@ -583,7 +618,8 @@ totem_pl_parser_parse_itms_doc (xml_node_t *item)
 		/* What we're looking for looks like:
 		 * <key>feedURL</key><string>URL</string> */
 		if (g_ascii_strcasecmp (item->name, "key") == 0
-		    && g_ascii_strcasecmp (item->data, "feedURL") == 0) {
+		    && g_ascii_strcasecmp (item->data, "feedURL") == 0
+		    && item->next != NULL) {
 			item = item->next;
 			if (g_ascii_strcasecmp (item->name, "string") == 0)
 				return item->data;
